package com.muyu.system.team.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.domain.ActivityTeamProductSkuInfo;
import com.muyu.domain.model.*;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.domain.ProjectSkuInfo;
import com.muyu.system.team.mapper.ActivityTeamProductSkuInfoMapper;
import com.muyu.system.team.service.ActivityTeamProductSkuInfoService;
import lombok.SneakyThrows;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.server.ServerCloneException;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
商品拼团规格信息表
 * @author: CuiFu Bo
 * @create: 2024-11-20 21:41
 **/


@Service
public class ActivityTeamProductSkuInfoServiceImpl extends ServiceImpl<ActivityTeamProductSkuInfoMapper, ActivityTeamProductSkuInfo> implements ActivityTeamProductSkuInfoService {

    @Resource
    private ProjectSkuCache projectSkuCache;

    @SneakyThrows
    @Override
    public TeamDiscountPriceModel getDiscountPrice(Long teamId) {
        List<ActivityTeamProductSkuInfo> productSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);
        if(productSkuInfoList.size()>0){


            Optional<TeamDiscountPriceModel> discountPrice = productSkuInfoList.stream()
                .map(activityTeamProductSkuInfo -> {
                    ProjectSkuInfo productPriceList = projectSkuCache.get(activityTeamProductSkuInfo.getProductId(), activityTeamProductSkuInfo.getProductSku());
                    return TeamDiscountPriceModel.of(productPriceList.getPrice(), activityTeamProductSkuInfo.getTeamPrice());
                }).min((o1, o2) -> Double.valueOf(o1.getDiscount() * 100 - o2.getDiscount() * 100).intValue());
        if (discountPrice.isEmpty()) {
            throw new ServerCloneException("团购活动下没有商品绑定");
        }
            return discountPrice.get();
        }
       return TeamDiscountPriceModel.builder()
               .productPrice(BigDecimal.valueOf(0))
               .teamPrice(BigDecimal.valueOf(0))
               .discount(0.00)
               .build();
    }

    @Override
    public TeamProductStockModel getStock(Long teamId) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);
        return TeamProductStockModel.builder()
                .teamStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getTeamStock).reduce(0L, Long::sum))
                .remainStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getRemainStock).reduce(0L, Long::sum))
                .build();
    }
    /**
     * 添加团购商品SKU
     *
     * @param activityTeamProductSkuAddModelList 团购商品SKU添加模型集合
     */
    @Override
    public void batchSave(List<ActivityTeamProductSkuAddModel> activityTeamProductSkuAddModelList) {
        if (activityTeamProductSkuAddModelList == null || activityTeamProductSkuAddModelList.isEmpty()) {
            throw new ServiceException("入参添加商品规格模型，不可为空");
        }
        this.saveBatch(
                activityTeamProductSkuAddModelList.stream().map(ActivityTeamProductSkuInfo::modelBuild).toList()
        );
    }

    /**
     * 通过拼团活动ID获取拼团下商品SKU集合
     *
     * @param teamId 活动ID
     * @return 拼团商品SKU集合
     */
    @Override
    public List<ActivityTeamProductSkuModel> findListByTeamId(Long teamId) {
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ActivityTeamProductSkuInfo::getTeamId,teamId);
        List<ActivityTeamProductSkuInfo> list = this.list(queryWrapper);
        return list.stream().map(ActivityTeamProductSkuModel::FindBuild).toList();
//        return list.stream().map(activityTeamProductSkuInfo -> ActivityTeamProductSkuModel.FindBuild(activityTeamProductSkuInfo)).toList();
    }

    @Transactional
    @Override
    public boolean updateBath(List<ActivityTeamProductSkuReqModel> activityTeamProductSkuReqModelList) {
        return this.updateBatchById(activityTeamProductSkuReqModelList.stream().map(ActivityTeamProductSkuInfo::updateModelBuild).toList());
    }

    /**
     * 根据 业务模型 进行 团购商品修改
     *
     * @param activityTeamProductSkuUpdModel 修改业务模型
     * @return 修改结果
     */
    @Override
    public boolean update(ActivityTeamProductSkuUpdModel activityTeamProductSkuUpdModel) {
        // update set  where
        LambdaUpdateWrapper<ActivityTeamProductSkuInfo> updateWrapper = new LambdaUpdateWrapper<>();
        // 字段=值
        updateWrapper.set(ActivityTeamProductSkuInfo::getTeamStock,activityTeamProductSkuUpdModel.getTeamStock());
        updateWrapper.set(ActivityTeamProductSkuInfo::getRemainStock,activityTeamProductSkuUpdModel.getTeamStock());
        updateWrapper.set(ActivityTeamProductSkuInfo::getTeamPrice,activityTeamProductSkuUpdModel.getTeamPrice());
        // id=？
        updateWrapper.eq(ActivityTeamProductSkuInfo::getId,activityTeamProductSkuUpdModel.getId());
        return this.update(updateWrapper);
    }

    /**
     * 根据 业务模型 进行 团购商品批量修改
     *
     * @param activityTeamProductSkuUpdModelList 修改业务模型
     * @return 修改结果
     */
    @Override
    public boolean batchUpdate(List<ActivityTeamProductSkuUpdModel> activityTeamProductSkuUpdModelList) {
        return this.updateBatchById(
                activityTeamProductSkuUpdModelList.stream()
                        .map(ActivityTeamProductSkuInfo::updModelBuild)
                        .toList()
        );
    }
}
